/* Test des Eval-Boards II
 * Anschlsse gem Eval-Board 2008
 * Getestet werden alle LEDs, das LCD und die serielle Schnittstelle:
 * 
 * LED-Test:     
 *   Sieht man
 * Display-Test: 
 *   4-Bit-Modus und 8-Bit Modus, erkennbar am Displaytext
 * Taster-Test:  
 *   Mit den drei Tastern lt sich der Inhalt des Displays verschieben 
 * RS232-Test:
 *   Terminal-Programm folgendermaen einstellen:
 *    - COM-Port
 *    - Bits pro Sekunde: 9600
 *    - Datenbits: 8
 *    - Pariatt: keine
 *    - Stoppbits: 1
 *    - Flusssteuerung: Kein
 *   Der Controller mu jedes gesendete Zeichen wieder zurcksenden!
 *
 * Das ist nur ein Testprogramm und dient nur als abschreckendes Beispiel 
 * - insbesondere die Interrupt-Annahme fr
 * die serielle Schnittstelle ist definitiv kein Zeichen fr guten
 * Programmierstil. 
 */


#ifndef F_CPU
  #define F_CPU 16000000UL
#endif

#include <avr/io.h>         
#include <util/delay.h>
#include <avr/interrupt.h>
#include <stdio.h>

#include "lcd_gemischt.h"
#include "usart.h"

char cs[40];
volatile int8_t lcdpos = 0;



// Receive-Interrupt
ISR(USART_RX_vect)
{
	while ( !(UCSR0A & (1 << RXC0) )); 
	char input = UDR0;                 
	while ( !(UCSR0A & (1 << UDRE0)) ); 
	UDR0 = input;                 


	if ( !(PINC & 0x01)) // Wenn Taster 1 gedrckt: HEX-ausgeben
	{
		lcd_pos4(0,1); 
		lcd_putstr4("                    "); // Mit Leerstring untere Zeile berschreiben
		lcd_pos4(0,1);
		snprintf(cs,40,"Z= 0x%X",input);
		lcd_putstr4(cs);
		lcd_pos4(0,1);
		lcdpos=0;
	}
	else
	{
		if (input == 0x0D)             // Return
		{
			lcd_pos4(0,1); 
			lcd_putstr4("                    "); // Mit Leerstring untere Zeile berschreiben
			lcd_pos4(0,1);
			lcdpos=0;
		}
		else if (input == 0x08)       // Backspace
		{
			lcdpos-=1;
			lcd_pos4(lcdpos,1);
		}
		else
		{
			lcd_putdat4(input);           // Zeichen auch auf LCD ausgeben
			lcdpos++;                     // Zeichenzhler erhhen
		}


		if ((lcdpos >= 20) | (lcdpos < 0) )
		{
			lcd_pos4(0,1);
			lcdpos = 0;
		}
	}
} // Ende Receive-Interrupt



inline void warte10ms(int faktor)
{
	for (int i=faktor; i>0; i--)
	{
		_delay_ms(10);
	}
}


void lauflicht(int delay)
{
  for (int i=0; i<=13; i++)
  {
  PORTB |= (1 << i);
  PORTD |= (1 << i) >> 6; 
  warte10ms(delay);
  }
}


int main(void)
{
  // Digitale Ausgnge:
	DDRB=0xff;
	DDRD=0xff;
  
  // Digitale Eingnge und Pullups:
	DDRC = 0x00;
	PORTC = (1 << PC0) | (1 << PC1) | (1 << PC2);

  // 1. Test: Lauflicht
	for (int j=3; j>0; j--) 
	{
    lauflicht(3);
		warte10ms(5);
	}

	warte10ms(50);

  // 2. Test: LCD 8-Bit-Modus
	lcd_init8();
	lcd_clear8();
	lcd_putstr8("LCD funktioniert");
	lcd_pos8(0,1);
	lcd_putstr8("8-Bit-Modus     ");
	warte10ms(50);
  PORTD=0x00;
	
  // 3. Test: LCD 4-Bit-Modus
  DDRD=0xF0;
	lcd_init4();
	lcd_clear4();
	lcd_putstr4("LCD funktioniert");
	lcd_pos4(0,1);
	lcd_putstr4("4-Bit-Modus     ");
	warte10ms(50);

  // 4. Test: Ankndigung: Taster werden aktiviert (in Hauptschleife) 
	lcd_clear4();
	lcd_putstr4("Die Taster k");
	lcd_putdat4(0xef);
	lcd_putstr4("nnen getestet werden.");
	lcd_pos4(0,1);

	warte10ms(200);
	
  // 5. Test: RS232 wird aktivert, durch Interrupt gehandhabt
  lcd_clear4();
	lcd_putstr4("RS232 9600N1 ");
	lcd_putstr4("kein Handshake");
	lcd_pos4(0,1);
  	
 	PORTD &= ~0x03;
	usart_init();
	lcd_pos4(0,1);

	usart_putstr("RS232 aktiv - zum Testen bitte etwas tippen!");

	warte10ms(1);
	sei();


  // Fr das Entprellen der Taster
	char taster0 = 0xFF;
	char taster1 = 0xFF;
	char taster2 = 0xFF;

  // Hauptschleife fragt nur die Taster ab
	while(1)
	{
		taster0 <<= 1;
		taster0 |= (PINC & 0x01);

		taster1 <<= 1;
		taster1 |= ((PINC & 0x02)>>1);

		taster2 <<= 1;
		taster2 |= ((PINC & 0x04)>>2);


		
		if ((taster0==0) && ((taster2!=0) || (taster1!=0)))
		{
			lcd_putcom4(0b00011000);
			taster0=0xff;
			while(!(PINC & (1 << PC0)));
		} 


		if (!(PINC & (1 << PC1)))
		{
			lcd_putcom4(0b00000010);
		}  


		if ((taster2==0) && ((taster0!=0) || (taster1!=0)))
		{
			lcd_putcom4(0b00011100);
			taster2=0xff;
			while(!(PINC & (1 << PC2)));
		}  

	}
	return 1;
}
