#include "lcd_gemischt.h"


inline void lcd_enablepulse()
{
  // Enable-Puls
  asm volatile ("nop");
  PORT_LCDC |= (1 << LCD_EN);
  asm volatile ("nop");
  PORT_LCDC &= ~(1 << LCD_EN);
}

/* LCD 4-Bit-Modus */

// Halbbytes ans LCD uebergeben
void lcd_putini4(char kommando)
{
  // Niederwertiges Halbbyte uebertragen
  // Niederwertiges Halbbyte uebertragen
  PORT_LCDD &= 0x0F;

  PORT_LCDD |= ((kommando << 4) & 0xF0);

  // Enable-Puls
  asm volatile ("nop");
  PORT_LCDC |= (1 << LCD_EN);
  asm volatile ("nop");
  asm volatile ("nop");
  asm volatile ("nop");
  PORT_LCDC &= ~(1 << LCD_EN);
  
  SHORTDELAY
  
}



// Kommandos an LCD uebergeben
void lcd_putcom4(char kommando)
{
  
  // Hoeherwertiges Halbbyte uebertragen
  PORT_LCDD &= 0x0F;
  PORT_LCDD |= (kommando & 0xF0);

  // Enable-Puls
  asm volatile ("nop");
  PORT_LCDC |= (1 << LCD_EN);
  asm volatile ("nop");
  asm volatile ("nop");
  asm volatile ("nop");
  PORT_LCDC &= ~(1 << LCD_EN);
  
  // Niederwertiges Halbbyte uebertragen
  PORT_LCDD &= 0x0F;
  PORT_LCDD |= ((kommando << 4) & 0xF0);



    // Enable-Puls
  asm volatile ("nop");
  PORT_LCDC |= (1 << LCD_EN);
  asm volatile ("nop");
  asm volatile ("nop");
  asm volatile ("nop");
  PORT_LCDC &= ~(1 << LCD_EN);
  
  SHORTDELAY
}

// Zeichen an LCD ausgeben
void lcd_putdat4(char zeichen)
{
  PORT_LCDC |=  (1 << LCD_RS); 
  lcd_putcom4(zeichen);
  PORT_LCDC &=  ~(1 << LCD_RS); 
}


// 4-Bit-Modus Initialisieren
void lcd_init4()
{
  // Pins auf Ausgang
  DDR_LCDD |= 0xf0;
  DDR_LCDC |= (1 << LCD_EN) | (1 << LCD_RW) | (1 << LCD_RS);

  // Pins auf Low
  PORT_LCDD = 0x00;
  PORT_LCDC &= ~((1 << LCD_EN) | (1 << LCD_RW) | (1 << LCD_RS));
  

  lcd_putini4(0x3);
  _delay_ms(5);
  lcd_putini4(0x3);
  _delay_ms(1);
  lcd_putini4(0x3);
  _delay_ms(1);
  lcd_putini4(0x2); 


  lcd_putcom4(0x0C);
  LONGDELAY
  lcd_putcom4(0x01);
  LONGDELAY
  lcd_putcom4(0x06);
  _delay_ms(5);

}

/* LCD 8-Bit-Modus */
// Kommandos an LCD uebergeben
void lcd_putcom8(char kommando)
{
  
  // Byte uebertragen
  PORT_LCDD = kommando;

  // Enable-Puls
  //asm volatile ("nop");
  //PORT_LCDC |= (1 << LCD_EN);
  //asm volatile ("nop");
  //asm volatile ("nop");
  //asm volatile ("nop");
  //PORT_LCDC &= ~(1 << LCD_EN);
  lcd_enablepulse();  

  SHORTDELAY
}

// Zeichen an LCD ausgeben
void lcd_putdat8(char zeichen)
{
  PORT_LCDC |=  (1 << LCD_RS); 
  lcd_putcom8(zeichen);
  PORT_LCDC &=  ~(1 << LCD_RS); 
}



// LCD initialisieren 8 Bit
void lcd_init8()
{
  // Pins auf Ausgang
  DDR_LCDD = 0xff;
  DDR_LCDC = (1 << LCD_EN) | (1 << LCD_RW) | (1 << LCD_RS);

  // Pins auf Low
  PORT_LCDD = 0x00;
  PORT_LCDC &= ~((1 << LCD_EN) | (1 << LCD_RW) | (1 << LCD_RS));

  lcd_putcom8(0x30);
  LONGDELAY
  lcd_putcom8(0x30);
  LONGDELAY
  lcd_putcom8(0x30);
  LONGDELAY

  lcd_putcom8(0b00111000);
  LONGDELAY
  lcd_putcom8(0x0C);
  LONGDELAY
  lcd_putcom8(0x01);
  LONGDELAY
  lcd_putcom8(0x06);
 
  _delay_ms(5); 
}






/*
 Kleine Sammlung von Hilfsfunktionen, um die Ausgabe an
 LCDs etwas komfortabler zu gestalten (nur 4 Bit-Modus)
 */

// Cursor auf Position X, Y bewegen   
void lcd_pos4(int posX, int posY)
{
  posX %= DISPLAYLENGTH;
  posY %= DISPLAYHEIGHT;
  lcd_putcom4((posX+posY*DISPADRESSABST) | 0x80);
  LONGDELAY
} 
// Cursor auf Position X, Y bewegen   
void lcd_pos8(int posX, int posY)
{
  posX %= DISPLAYLENGTH;
  posY %= DISPLAYHEIGHT;
  lcd_putcom8((posX+posY*DISPADRESSABST) | 0x80);
  LONGDELAY
} 



// Inhalt des Displays komplett loeschen 
void lcd_clear4()
{
  lcd_putcom4(CLEARCOMMAND);
  LONGDELAY
}
// Inhalt des Displays komplett loeschen 
void lcd_clear8()
{
  lcd_putcom8(CLEARCOMMAND);
  LONGDELAY
}



// Strings auf LCD ausgeben
void lcd_putstr4(char *zeichenkette)
{
  while (*zeichenkette)
  {
    lcd_putdat4(*zeichenkette);
    zeichenkette++;
  }
}

// Strings auf LCD ausgeben
void lcd_putstr8(char *zeichenkette)
{
  while (*zeichenkette)
  {
    lcd_putdat8(*zeichenkette);
    zeichenkette++;
  }
}


