/* Funktionen fuer die Ansteuerung von HD44780-kompatiblen
 * LCDs im 4-Bit-Modus
 * Delay-Funktionen entfernen
 */


#include <avr/io.h>
#include <util/delay.h>
#include "lcd4.h" 

inline void lcd_enablepulse()
{
  // Enable-Puls
  asm volatile ("nop");
  PORT_LCDC |=  (1 << LCD_EN);
  asm volatile ("nop");
  PORT_LCDC &= ~(1 << LCD_EN);
}


inline void lcd_waitwhilebusy()
{
    asm volatile ("nop");
      asm volatile ("nop");
        asm volatile ("nop");  


  // Busy-Pin auf Eingang; Lesen signalisieren
  DDR_LCDD  &= ~(1 << DB3_LCDD);
  PORT_LCDC |=  (1 << LCD_RW); 
   
    asm volatile ("nop");
      asm volatile ("nop");
  while (bit_is_set(PIN_LCDD, DB3_LCDD));
  
  // Wenn Busy-Flag weg wieder auf Ausgang und Schreiben
  PORT_LCDC &= ~(1 << LCD_RW); 
  DDR_LCDD  |=  (1 << DB3_LCDD);

      asm volatile ("nop");
      asm volatile ("nop");
        asm volatile ("nop");
}

// Halbbytes ans LCD uebergeben
void lcd_putini(char kommando)
{
  // Niederwertiges Halbbyte uebertragen
  PORT_LCDD &= ~(0x0F << DB0_LCDD);
  PORT_LCDD |= (kommando & 0x0F) << DB0_LCDD;
  
  lcd_enablepulse();
  
  SHORTDELAY
}



// Kommandos an LCD uebergeben
void lcd_putcom(char kommando)
{
  // Hoeherwertiges Halbbyte uebertragen
  PORT_LCDD &= ~(0x0F << DB0_LCDD);
  PORT_LCDD |= ((kommando & 0xF0) >> 4) << DB0_LCDD;

  lcd_enablepulse();
  
  // Niederwertiges Halbbyte uebertragen
  PORT_LCDD &= ~(0x0F << DB0_LCDD);
  PORT_LCDD |= (kommando & 0x0F) << DB0_LCDD;

  lcd_enablepulse();
  
  SHORTDELAY
}

// Zeichen an LCD ausgeben
void lcd_putchar(char zeichen)
{
  PORT_LCDC |=  (1 << LCD_RS); 
  lcd_putcom(zeichen);
  PORT_LCDC &=  ~(1 << LCD_RS); 
}


// LCD initialisieren 4 Bit
void lcd_init()
{
  // Pins auf Ausgang
  DDR_LCDD |= 0x0F << DB0_LCDD;
  DDR_LCDC |= (1 << LCD_EN) | (1 << LCD_RW) | (1 << LCD_RS);

  // Pins auf Low
  PORT_LCDD &= ~(0x0F << DB0_LCDD);
  PORT_LCDC &= ~((1 << LCD_EN) | (1 << LCD_RW) | (1 << LCD_RS));
  
  // Initialisierung starten
  lcd_putini(0x3);
  _delay_ms(5);
  lcd_putini(0x3);
  _delay_ms(1);
  lcd_putini(0x3);
  _delay_ms(1);
  lcd_putini(0x2); 

  // Ab jetzt ist das LCD im 4-Bit-Modus und es koennen ganze
  // Kommandos gesendet werden
  lcd_putcom(0x0C);
  LONGDELAY
  lcd_putcom(0x01);
  LONGDELAY
  lcd_putcom(0x06);

  // Noch ein wenig warten
  _delay_ms(5);
}
